@@ -5,7 +5,7 @@ use warnings;
 package LINQ::Collection;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_006';
+our $VERSION   = '0.000_007';
 
 use Role::Tiny;
 use LINQ::Util::Internal ();
@@ -34,26 +34,85 @@ my $_coerce = sub {
 sub select {
 	my $self = shift;
 	my $map  = LINQ::Util::Internal::assert_code( @_ );
-	LINQ::Util::Internal::create_linq(
-		[ map scalar( $map->( $_ ) ), $self->to_list ],
+	
+	my $iter = $self->to_iterator;
+	my $stopped;
+	
+	require LINQ;
+	LINQ::LINQ(
+		sub {
+			return LINQ::END() if $stopped;
+			my @got = $iter->();
+			if ( @got ) {
+				local $_;
+				return scalar $map->( $_ = $got[0] );
+			}
+			++$stopped;
+			return LINQ::END();
+		}
 	);
-}
+} #/ sub select
 
 sub where {
 	my $self   = shift;
 	my $filter = LINQ::Util::Internal::assert_code( @_ );
-	LINQ::Util::Internal::create_linq(
-		[ grep $filter->( $_ ), $self->to_list ],
+	
+	my $iter = $self->to_iterator;
+	my $stopped;
+	
+	require LINQ;
+	LINQ::LINQ(
+		sub {
+			GETVAL: {
+				return LINQ::END() if $stopped;
+				my @got = $iter->();
+				if ( @got ) {
+					local $_;
+					my $pass = $filter->( $_ = $got[0] );
+					return $got[0] if $pass;
+					redo GETVAL;
+				}
+				++$stopped;
+				return LINQ::END();
+			} #/ GETVAL:
+		}
 	);
-}
+} #/ sub where
 
 sub select_many {
 	my $self = shift;
 	my $map  = LINQ::Util::Internal::assert_code( @_ );
-	LINQ::Util::Internal::create_linq(
-		[ map $map->( $_ )->$_coerce->to_list, $self->to_list ],
+	
+	my $outer = $self->to_iterator;
+	my $inner;
+	my $end;
+	
+	require LINQ;
+	LINQ::LINQ(
+		sub {
+			BODY: {
+				return LINQ::END() if $end;
+				if ( not $inner ) {
+					$inner = $outer->();
+					if ( defined $inner ) {
+						local $_;
+						$inner = $map->( $_ = $inner )->$_coerce->to_iterator;
+					}
+					else {
+						$end = 1;
+						return LINQ::END();
+					}
+				} #/ if ( not $inner )
+				my @got = $inner->();
+				if ( not @got ) {
+					undef $inner;
+					redo BODY;
+				}
+				return @got;
+			} #/ BODY:
+		}
 	);
-}
+} #/ sub select_many
 
 sub min {
 	my $self = shift;
@@ -98,16 +157,17 @@ my $_prepare_join = sub {
 	my $y_keys = LINQ::Util::Internal::assert_code( shift );
 	my $joiner = LINQ::Util::Internal::assert_code( @_ );
 	
-	$hint =~ /\A-(inner|left|right|outer)\z/ or LINQ::Util::Internal::throw(
+	$hint =~ /\A-(inner|left|right|outer)\z/
+		or LINQ::Util::Internal::throw(
 		"CallerError",
 		message => "Expected a recognized join type; got '$hint'"
-	);
-	
+		);
+		
 	my @x_mapped =
 		$x->select( sub { [ scalar( $x_keys->( $_[0] ) ), $_[0] ] } )->to_list;
 	my @y_mapped =
 		$y->select( sub { [ scalar( $y_keys->( $_[0] ) ), $_[0] ] } )->to_list;
-	
+		
 	return ( \@x_mapped, \@y_mapped, $hint, $joiner );
 };
 
@@ -193,15 +253,17 @@ sub take_while {
 	my $iter    = $self->to_iterator;
 	
 	require LINQ;
-	LINQ::Util::Internal::create_linq( sub {
-		return LINQ::END() if $stopped;
-		my @got = $iter->();
-		if ( ! @got or ! $filter->( $_ = $got[0] ) ) {
-			$stopped++;
-			return LINQ::END();
+	LINQ::LINQ(
+		sub {
+			return LINQ::END() if $stopped;
+			my @got = $iter->();
+			if ( !@got or !$filter->( $_ = $got[0] ) ) {
+				$stopped++;
+				return LINQ::END();
+			}
+			return $got[0];
 		}
-		return $got[0];
-	} );
+	);
 } #/ sub take_while
 
 sub skip {
@@ -218,22 +280,24 @@ sub skip_while {
 	my $iter    = $self->to_iterator;
 	
 	require LINQ;
-	LINQ::Util::Internal::create_linq( sub {
-		SKIPPING: {
-			return LINQ::END() if $stopped;
-			my @got = $iter->();
-			if ( ! @got ) {
-				$stopped++;
-				redo SKIPPING;
-			}
-			return $got[0] if $started;
-			if ( $filter->( $_ = $got[0] ) ) {
-				redo SKIPPING;
-			}
-			++$started;
-			return $got[0];
-		};
-	} );
+	LINQ::LINQ(
+		sub {
+			SKIPPING: {
+				return LINQ::END() if $stopped;
+				my @got = $iter->();
+				if ( !@got ) {
+					$stopped++;
+					redo SKIPPING;
+				}
+				return $got[0] if $started;
+				if ( $filter->( $_ = $got[0] ) ) {
+					redo SKIPPING;
+				}
+				++$started;
+				return $got[0];
+			} #/ SKIPPING:
+		}
+	);
 } #/ sub skip_while
 
 sub concat {
@@ -241,38 +305,38 @@ sub concat {
 	my $idx = 0;
 	
 	require LINQ;
-	LINQ::Util::Internal::create_linq( sub {
-		FIND_NEXT: {
-			return LINQ::END() if not @collections;
-			
-			my @got = $collections[0]->();
-			if ( not @got ) {
-				shift @collections;
-				redo FIND_NEXT;
-			}
-			
-			return $got[0];
-		};
-	} );
-}
+	LINQ::LINQ(
+		sub {
+			FIND_NEXT: {
+				return LINQ::END() if not @collections;
+				
+				my @got = $collections[0]->();
+				if ( not @got ) {
+					shift @collections;
+					redo FIND_NEXT;
+				}
+				
+				return $got[0];
+			} #/ FIND_NEXT:
+		}
+	);
+} #/ sub concat
 
 sub order_by {
 	my $self   = shift;
 	my $hint   = ref( $_[0] ) ? -numeric : shift( @_ );
-	my $keygen = @_ ? LINQ::Util::Internal::assert_code( @_ ) : undef;
+	my $keygen = @_           ? LINQ::Util::Internal::assert_code( @_ ) : undef;
 	
 	if ( not $keygen ) {
 		if ( $hint eq -string ) {
 			return LINQ::Util::Internal::create_linq(
-				[ sort { $a cmp $b } $self->to_list ]
-			);
+				[ sort { $a cmp $b } $self->to_list ] );
 		}
 		elsif ( $hint eq -numeric ) {
 			return LINQ::Util::Internal::create_linq(
-				[ sort { $a <=> $b } $self->to_list ]
-			);
+				[ sort { $a <=> $b } $self->to_list ] );
 		}
-	}
+	} #/ if ( not $keygen )
 	
 	if ( $hint eq -string ) {
 		return LINQ::Util::Internal::create_linq(
@@ -351,9 +415,10 @@ sub group_by {
 } #/ sub group_by
 
 sub distinct {
-	my $self    = shift;
-	my $compare = @_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
-	
+	my $self = shift;
+	my $compare =
+		@_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
+		
 	my @already;
 	$self->where(
 		sub {
@@ -374,16 +439,18 @@ sub union {
 }
 
 sub intersect {
-	my $self    = shift;
-	my $other   = shift;
-	my @compare = @_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
+	my $self  = shift;
+	my $other = shift;
+	my @compare =
+		@_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
 	$self->where( sub { $other->contains( $_, @compare ) } );
 }
 
 sub except {
-	my $self    = shift;
-	my $other   = shift;
-	my @compare = @_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
+	my $self  = shift;
+	my $other = shift;
+	my @compare =
+		@_ ? LINQ::Util::Internal::assert_code( @_ ) : sub { $_[0] == $_[1] };
 	$self->where( sub { not $other->contains( $_, @compare ) } );
 }
 
@@ -391,24 +458,37 @@ sub sequence_equal {
 	my $self = shift;
 	my ( $other, @compare ) = @_;
 	
-	return !!0 unless $self->count == $other->count;
-	
-	my @list1 = $self->to_list;
-	my @list2 = $other->to_list;
-	return !!0 unless @list1 == @list2;
-	
+	my $compare;
 	if ( @compare ) {
-		my $compare = LINQ::Util::Internal::assert_code( @compare );
-		for my $i ( 0 .. $#list1 ) {
-			return !!0 unless $compare->( $list1[$i], $list2[$i] );
-		}
-		return !!1;
+		$compare = LINQ::Util::Internal::assert_code( @compare );
 	}
 	
-	for my $i ( 0 .. $#list1 ) {
-		return !!0 unless $list1[$i] == $list2[$i];
-	}
-	return !!1;
+	my $iter1 = $self->to_iterator;
+	my $iter2 = $other->to_iterator;
+	
+	while ( 1 ) {
+		my @got1 = $iter1->();
+		my @got2 = $iter2->();
+		
+		if ( not @got1 ) {
+			if ( @got2 ) {
+				return !!0;
+			}
+			else {
+				return !!1;
+			}
+		}
+		elsif ( not @got2 ) {
+			return !!0;
+		}
+		
+		if ( $compare ) {
+			return !!0 unless $compare->( $got1[0], $got2[0] );
+		}
+		else {
+			return !!0 unless $got1[0] == $got2[0];
+		}
+	} #/ while ( 1 )
 } #/ sub sequence_equal
 
 my $_with_default = sub {
@@ -467,7 +547,8 @@ sub single {
 	return $found->element_at( 0 ) if $found->count == 1;
 	$found->count == 0
 		? LINQ::Util::Internal::throw( 'NotFound', collection => $self )
-		: LINQ::Util::Internal::throw( 'MultipleFound', collection => $self, found => $found );
+		: LINQ::Util::Internal::throw( 'MultipleFound', collection => $self,
+		found => $found );
 }
 
 sub single_or_default {
@@ -497,14 +578,17 @@ sub element_at_or_default {
 
 sub any {
 	my $self = shift;
-	@_
-		? $self->where( @_ )->any
-		: ( $self->count > 0 );
+	my $iter = @_ ? $self->where( @_ )->to_iterator : $self->to_iterator;
+	my @got  = $iter->();
+	!!scalar @got;
 }
 
 sub all {
-	my $self = shift;
-	$self->where( @_ )->count == $self->count;
+	my $self  = shift;
+	my $check = LINQ::Util::Internal::assert_code( @_ );
+	my $iter  = $self->where( sub { not $check->( $_ ) } )->to_iterator;
+	my @got   = $iter->();
+	!scalar @got;
 }
 
 sub contains {
@@ -516,7 +600,11 @@ sub contains {
 		return $self->any( LINQ::Util::Internal::assert_code( @args ) );
 	}
 	
-	$self->any( sub { $_ == $x } );
+	my $iter = $self->to_iterator;
+	while ( 1 ) {
+		my @got = $iter->() or return !!0;
+		return !!1 if $got[0] == $x;
+	}
 } #/ sub contains
 
 sub count {
@@ -601,12 +689,14 @@ sub zip {
 	my @results;
 	
 	require LINQ;
-	LINQ::Util::Internal::create_linq( sub {
-		my @r1 = $iter1->();
-		my @r2 = $iter2->();
-		return LINQ::END unless @r1 && @r2;
-		$map->( $r1[0], $r2[0] );
-	} );
+	LINQ::LINQ(
+		sub {
+			my @r1 = $iter1->();
+			my @r2 = $iter2->();
+			return LINQ::END unless @r1 && @r2;
+			$map->( $r1[0], $r2[0] );
+		}
+	);
 } #/ sub zip
 
 sub default_if_empty {
@@ -614,7 +704,7 @@ sub default_if_empty {
 	my $item = shift;
 	
 	if ( $self->count == 0 ) {
-		return LINQ::Util::Internal::create_linq( [ $item ] );
+		return LINQ::Util::Internal::create_linq( [$item] );
 	}
 	
 	return $self;
@@ -637,7 +727,7 @@ sub foreach {
 		die( $e ) unless $e->isa( 'LINQ::LAST' );
 	}
 	return;
-}
+} #/ sub foreach
 
 1;
 
@@ -1243,6 +1333,23 @@ method is provided by the MoreLINQ extension.
 
 =back
 
+=head1 WORKING WITH INFINITE COLLECTIONS
+
+Because LINQ collections can be instantiated from an iterator, they may
+contain infinite items.
+
+Certain methods aggregate the entire collection, so can go into an infinite
+loop. This includes: C<aggregate>, C<min>, C<max>, C<sum>, C<average>, and
+C<count>.
+
+Other methods which will go into an infinite loop on infinite collections:
+C<join>, C<group_join>, C<group_by>, C<order_by>, C<order_by_descending>,
+C<reverse>, C<to_lookup>, C<to_dictionary>, and C<to_list>.
+
+The C<to_array> method in general I<will> succeed on infinite collections
+as it can return a reference to a tied array. However, trying to dereference
+the entire array to a list will fail.
+
 =head1 BUGS
 
 Please report any bugs to
@@ -1270,4 +1377,3 @@ the same terms as the Perl 5 programming language system itself.
 THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-
