@@ -90,11 +90,10 @@ if ( $] < 5.010000 ) {
 				and ( $self->[__EXHAUSTED] = !!1 )
 				and pop( @$cache )
 				and (
-					@got
-						? $self->$_throw_caller_error( 'Returned values after LINQ::END' )
-						: return ()
-				)
-				while @got;
+				@got
+				? $self->$_throw_caller_error( 'Returned values after LINQ::END' )
+				: return ()
+				) while @got;
 				
 			redo EXTEND;
 		} #/ EXTEND:
@@ -104,9 +103,9 @@ if ( $] < 5.010000 ) {
 package LINQ::Iterator;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_006';
+our $VERSION   = '0.000_007';
 
-use Role::Tiny::With ();
+use Role::Tiny::With     ();
 use LINQ::Util::Internal ();
 
 Role::Tiny::With::with( qw( LINQ::Collection ) );
@@ -114,14 +113,15 @@ Role::Tiny::With::with( qw( LINQ::Collection ) );
 sub new {
 	my $class = shift;
 	if ( @_ ) {
-		tie my ( @arr ), 'LINQ::Iterator::_LazyList', LINQ::Util::Internal::assert_code( @_ );
+		tie my ( @arr ), 'LINQ::Iterator::_LazyList',
+			LINQ::Util::Internal::assert_code( @_ );
 		return bless \@arr, $class;
 	}
 	LINQ::Util::Internal::throw(
 		"CallerError",
 		message => "Expected a coderef"
 	);
-}
+} #/ sub new
 
 sub _guts {
 	my $self = shift;
@@ -157,49 +157,6 @@ sub element_at {
 	$self->LINQ::Collection::element_at( @_ );
 } #/ sub element_at
 
-sub select {
-	my $self = shift;
-	
-	if ( my $guts = $self->_guts ) {
-		my $map = LINQ::Util::Internal::assert_code( @_ );
-		my $idx = 0;
-		return LINQ::Util::Internal::create_linq( sub {
-			my $val = $guts->fetch_ref( $idx++ );
-			if ( ! $val ) {
-				require LINQ;
-				return LINQ::END();
-			}
-			local $_ = $$val;
-			scalar $map->( $_ );
-		} );
-	}
-	
-	$self->LINQ::Collection::select( @_ );
-}
-
-sub where {
-	my $self = shift;
-	
-	if ( my $guts = $self->_guts ) {
-		my $check = LINQ::Util::Internal::assert_code( @_ );
-		my $idx   = 0;
-		return LINQ::Util::Internal::create_linq( sub {
-			GETVAL: {
-				my $val = $guts->fetch_ref( $idx++ );
-				if ( ! $val ) {
-					require LINQ;
-					return LINQ::END();
-				}
-				local $_ = $$val;
-				redo GETVAL unless $check->( $_ );
-				return $$val;
-			};
-		} );
-	}
-	
-	$self->LINQ::Collection::where( @_ );
-}
-
 sub to_iterator {
 	my $self = shift;
 	
@@ -213,9 +170,9 @@ sub to_iterator {
 			++$done;
 			return;
 		};
-	}
+	} #/ if ( my $guts = $self->...)
 	
 	$self->LINQ::Collection::to_iterator( @_ );
-}
+} #/ sub to_iterator
 
 1;
